<?php

/**
 * Run the provied IP address against the list of ignored IPs
 *
 * @ver 0.1.0
 * @param int $ip The IP address to be checked
 * @return bool True on match, false otherwise
 */

function kstats_ignore_ip( $ip  = '') {

	if ( empty( $ip ) || ! file_exists( KSTATS_FILE_IGNORED_IPS ) )
		return FALSE;

	$lines = file( KSTATS_FILE_IGNORED_IPS );

	foreach ( $lines as $line ) {
		$line = rtrim( $line, "\r\n" );
		if ( preg_match( "~$line~", $ip ) )
		   return TRUE;
	}

	return FALSE;

}


/**
 * Grab the requested URL and format it for use by the collector. Nothing
 * amazing going on here, the function will remove the trailing slash and
 * query delimiter on non-permalinked sites or for searches. Adds the
 * fqdn when the root of the site is accessed.
 *
 * @ver 0.1.0
 * @return string The formatted request URI
 */

function kstats_get_url () {

	$url = esc_url( $_SERVER['REQUEST_URI'] );

	if ( substr ( $url, 0, 2) == '/?' )
		$url = substr ( $url, 2 );

	if ( $url == '/' )
		$url = 'http://' . $_SERVER['HTTP_HOST'] . '/';

	return $url;
}


/**
 * Attempts to determine if our visitor is a search engine spider or not
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $user_agent The user agent string to run the query against
 * @return FALSE|string Returns the name of the spider if matched, false otherwise
 */

function kstats_trap_spider( $user_agent = '' ) {

	if ( empty( $user_agent ) || ! file_exists( KSTATS_FILE_SPIDERS ) )
		return FALSE;

	$lines = file( KSTATS_FILE_SPIDERS );

	foreach ( $lines as $line ) {
		list( $nome, $id ) = explode( '|', $line );
		if ( strpos( '+' . $user_agent, $id ) )
			return $nome;
	}

	return FALSE;
}


/**
 * Attempts to determine whether or not the visitor is accessing a feed, and which type
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $url The url string to run the query against
 * @return FALSE|string Returns the type of feed if matched, false otherwise
 *
 * @todo Not very happy with the use of get_bloginfo for this, need to find a better method
 */

function kstats_trap_feed( $url = '' ) {

	if ( empty( $url ) || strpos( '+' . $url, 'feed' ) === FALSE )
		return FALSE;

	if ( stristr( $url, get_bloginfo( 'comments_atom_url' ) ) != FALSE )
		return 'COMMENT ATOM';
	else if ( stristr( $url, get_bloginfo( 'comments_rss2_url' ) ) != FALSE )
		return 'COMMENT RSS';
	else if ( stristr( $url, get_bloginfo( 'rdf_url' ) ) != FALSE )
		return 'RDF';
	else if ( stristr( $url, get_bloginfo( 'atom_url' ) ) != FALSE )
		return 'ATOM';
	else if ( stristr( $url, get_bloginfo( 'rss_url' ) ) != FALSE )
		return 'RSS';
	else if ( stristr( $url, get_bloginfo( 'rss2_url' ) ) != FALSE )
		return 'RSS2';
	elseif ( stristr( $url, 'wp-feed.php' ) != FALSE )
		return 'RSS2';
	else if ( stristr( $url, '/feed' ) != FALSE )
		return 'RSS2';

	return FALSE;
}


/**
 * Attempts to determine the visitors Operating System based on their User Agent
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $user_agent The user agent string to run the query against
 * @return FALSE|string Returns the name of their OS if matched, false otherwise
 */

function kstats_trap_os( $user_agent = '' ) {

	if ( empty( $user_agent ) || ! file_exists( KSTATS_FILE_OS ) )
		return FALSE;

	$lines = file( KSTATS_FILE_OS );

	$user_agent = str_replace( ' ', '', $user_agent ); // remove spaces to match against the ID in our definition list

	foreach ( $lines as $line ) {
		list( $nome, $id) = explode( '|', $line );
		if ( strpos( '+' . $user_agent, $id ) !== FALSE )
			return $nome;
	}

	return FALSE;
}


/**
 * Attempts to determine the visitors Browser based on their User Agent
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $user_agent The user agent string to run the query against
 * @return FALSE|string Returns the name of their browser if matched, false otherwise
 */

function kstats_trap_browser ( $user_agent = '' ) {

	if ( empty( $user_agent ) || ! file_exists( KSTATS_FILE_BROWSERS ) )
		return FALSE;

	$lines = file( KSTATS_FILE_BROWSERS );

	$user_agent = str_replace( ' ', '', $user_agent ); // remove spaces to match against the ID in our definition list

	foreach ( $lines as $line ) {
		list ( $nome, $id ) = explode ( '|', $line );
		if ( strpos ( '+' . $user_agent, $id ) !== FALSE )
			return $nome;
	}

	return FALSE;
}


/**
 * Attempts to determine any search terms used to find the site, based on the referrer
 *
 * @ver 0.1.0
 * @category Trap
 * @uses kstats_trap_search_terms
 * @param string $referrer The referrer string to run the query against
 * @return FALSE|string Returns the search terms if trapped, false otherwise
 */

function kstats_trap_search_query( $referrer = '' ) {

	if ( empty( $referrer ) || ! file_exists( KSTATS_FILE_SEARCH_QUERIES ) )
		return FALSE;

	$lines = file( KSTATS_FILE_SEARCH_QUERIES );

	foreach ( $lines as $line) {
		list( $nome, $url, $id ) = explode( '|', $line );

		// Is our referrer coming from a recognized search engine?
		if ( strpos( '+' . $referrer, $url ) !== FALSE ) {

			$queries = kstats_trap_search_terms( $referrer );
			$i = count( $queries );

			while ( $i-- ) {
				$parts = explode( '=', $queries[$i] );
				if ( $parts[0] == $id )
					return ( $nome . '|' . urlencode ( $parts[1] ) );
			}

		} // if

	} // foreach

	return FALSE;
}


/**
 * Used by kstats_trap_search_query to pull the query string apart and perform a little kung-fu on it
 *
 * @ver 0.1.0
 * @category Trap
 * @param string $url The url string to run the query against
 * @return FALSE|array Returns an array of parts from the query string, false otherwise
 */

function kstats_trap_search_terms( $url ) {

	$query_str = parse_url( $url, PHP_URL_QUERY );

	if ( ! empty( $query_str ) )
		return explode( '&', html_entity_decode( urldecode( $query_str ) ) );

	return FALSE;
}


/**
 * Nothing special, simply returns the string with an ellipsis on the end
 * if it exceeds the requested length.
 *
 * @ver 0.1.0
 * @param string $string The string to truncate
 * @param int    $length The maximum length allowable in characters
 * @return string The original string, formatted if it exceeded $length, the same otherwise
 */

function kstats_truncate( $string, $length ) {
	if ( $length > strlen( $string ) )
		return $string;

	return mb_substr( $string, 0, $length ) . '...';
}


/**
 * Pulls data from the kStats totals table for use in building the overview table
 * and accompanying graphical bar chart.
 *
 * @ver 0.1.0
 * @category Database
 * @return array An array of data, including the charts days, totals, and $max for use in building the chart
 */

function kstats_get_chart_totals () {
	global $wpdb, $kstatstime;


	$query = sprintf ( "SELECT type, name, total, last, this, yesterday FROM `%s`", KSTATS_TABLE_TOTALS );
	$results = $wpdb->get_results ( $query, ARRAY_A );

	// Initialize our values
	$totals = array();
	$charts = array();
	$max = 0;

	if ( is_array( $results ) ) {
		// and turn it into an easy to use array inside of one query as opposed to our predecessors 104
		foreach ( $results as $row ) {
			if ( 'charts' == $row['type'] ) {
				$charts[$row['name']]['visitors']  = $row['total'];
				$charts[$row['name']]['pageviews'] = $row['last'];
				$charts[$row['name']]['spiders']   = $row['this'];
				$charts[$row['name']]['feeds']     = $row['yesterday'];

				if ( ! isset( $last_day ) )
					$last_day = $row['name'];
				else if ( $row['name'] > $last_day )
					$last_day = $row['name'];

				$total = $row['total'] + $row['last'] + $row['this'] + $row['yesterday'];
				if ( $total > $max )
					$max = $total;
			}
			else {
				$totals[$row['name']]['total']     = $row['total'];
				$totals[$row['name']]['last']      = $row['last'];
				$totals[$row['name']]['this']      = $row['this'];
				$totals[$row['name']]['yesterday'] = $row['yesterday'];
			}
		}


		if ( $last_day == $kstatstime->today ) {
			$totals['visitors']['today']  = $charts[$kstatstime->today]['visitors'];
			$totals['pageviews']['today'] = $charts[$kstatstime->today]['pageviews'];
			$totals['spiders']['today']   = $charts[$kstatstime->today]['spiders'];
			$totals['feeds']['today']     = $charts[$kstatstime->today]['feeds'];
		}
		else if ( $last_day == $kstatstime->yesterday ) {
			$totals['visitors']['yesterday']  = $charts[$kstatstime->yesterday]['visitors'];
			$totals['pageviews']['yesterday'] = $charts[$kstatstime->yesterday]['pageviews'];
			$totals['spiders']['yesterday']   = $charts[$kstatstime->yesterday]['spiders'];
			$totals['feeds']['yesterday']     = $charts[$kstatstime->yesterday]['feeds'];

			$totals['visitors']['today']  = 0;
			$totals['pageviews']['today'] = 0;
			$totals['spiders']['today']   = 0;
			$totals['feeds']['today']     = 0;
		}

	} // if ( is_array )

	// The following chunk needs to be rolled up into the 'today' collection, this is simply a stopgap to get the darn plugin out. I know, bad.
	$query = sprintf ( "SELECT COUNT( DISTINCT ip ) AS visitors, COUNT( timestamp ) AS pageviews FROM `%s` WHERE feed = '' AND spider = '' AND SUBSTR( timestamp, 1, 10 ) = '%s' AND preserved IS NULL", KSTATS_TABLE_RAW, $kstatstime->yesterday );
	$result = $wpdb->get_row ( $query );

	$charts[$kstatstime->yesterday]['visitors'] += $result->visitors;
	$charts[$kstatstime->yesterday]['pageviews'] += $result->pageviews;
	
	$totals['visitors']['yesterday']  += $result->visitors;
	$totals['pageviews']['yesterday'] += $result->pageviews;


	$query = sprintf ( "SELECT COUNT( ip ) AS spiders FROM `%s` WHERE spider <> '' AND SUBSTR( timestamp, 1, 10) = '%s' AND preserved IS NULL", KSTATS_TABLE_RAW, $kstatstime->yesterday );
	$results = $wpdb->get_row( $query );

	$charts[$kstatstime->yesterday]['spiders'] += $results->spiders;
	$totals['spiders']['yesterday'] += $results->spiders;


	$query = sprintf ( "SELECT COUNT( ip ) AS feeds FROM `%s` WHERE feed <> '' AND SUBSTR(timestamp, 1, 10) = '%s' AND preserved IS NULL", KSTATS_TABLE_RAW, $kstatstime->yesterday );
	$results = $wpdb->get_row( $query );

	$charts[$kstatstime->yesterday]['feeds'] += $results->feeds;
	$totals['feeds']['yesterday'] += $results->feeds;


	$total = $charts[$kstatstime->yesterday]['visitors'] + $charts[$kstatstime->yesterday]['pageviews'] + $charts[$kstatstime->yesterday]['spiders'] + $charts[$kstatstime->yesterday]['feeds'];

	if ( $total > $max )
		$max = $total;


	// Now that we have the aggregated totals, we need to include current data as well
	$query = sprintf ( "SELECT COUNT( DISTINCT ip ) AS visitors, COUNT( timestamp ) AS pageviews FROM `%s` WHERE feed = '' AND spider = '' AND SUBSTR( timestamp, 1, 10 ) = '%s' AND preserved IS NULL", KSTATS_TABLE_RAW, $kstatstime->today );
	$result = $wpdb->get_row ( $query );

	$charts[$kstatstime->today]['visitors'] += $result->visitors;
	$charts[$kstatstime->today]['pageviews'] += $result->pageviews;
	
	$totals['visitors']['today']  += $result->visitors;
	$totals['pageviews']['today'] += $result->pageviews;

	$totals['visitors']['total'] += $totals['visitors']['today'];
	$totals['visitors']['this']  += $totals['visitors']['today'];

	$totals['pageviews']['total'] += $totals['pageviews']['today'];
	$totals['pageviews']['this']  += $totals['pageviews']['today'];

	$query = sprintf ( "SELECT COUNT( ip ) AS spiders FROM `%s` WHERE spider <> '' AND SUBSTR( timestamp, 1, 10) = '%s' AND preserved IS NULL", KSTATS_TABLE_RAW, $kstatstime->today );
	$results = $wpdb->get_row( $query );

	$charts[$kstatstime->today]['spiders'] += $results->spiders;
	$totals['spiders']['today'] += $results->spiders;

	$totals['spiders']['total'] += $totals['spiders']['today'];
	$totals['spiders']['this']  += $totals['spiders']['today'];

	$query = sprintf ( "SELECT COUNT( ip ) AS feeds FROM `%s` WHERE feed <> '' AND SUBSTR(timestamp, 1, 10) = '%s' AND preserved IS NULL", KSTATS_TABLE_RAW, $kstatstime->today );
	$results = $wpdb->get_row( $query );

	$charts[$kstatstime->today]['feeds'] += $results->feeds;
	$totals['feeds']['today'] += $results->feeds;

	$totals['feeds']['total'] += $totals['feeds']['today'];
	$totals['feeds']['this']  += $totals['feeds']['today'];

	$total = $charts[$kstatstime->today]['visitors'] + $charts[$kstatstime->today]['pageviews'] + $charts[$kstatstime->today]['spiders'] + $charts[$kstatstime->today]['feeds'];

	if ( $total > $max )
		$max = $total;

	// Make sure the charts are sorted - is this redundant? They usually are anyways
	uksort ( $charts, create_function ( '$a, $b', 'if ( strtotime ( $a ) == strtotime ( $b ) ) return 0; else return ( strtotime ( $a ) < strtotime ( $b ) ) ? -1 : 1;' ) );

	// Return the results as an array for extraction
	return array( 'charts' => $charts, 'totals' => $totals, 'max' => $max );
}